# -*- coding: utf-8 -*-
from AccessControl.unauthorized import Unauthorized
from design.plone.contenttypes.controlpanels.settings import (
    IDesignPloneSettings,
)
from plone import api
from plone.registry.interfaces import IRegistry
from plone.restapi.interfaces import ISerializeToJsonSummary
from plone.restapi.services import Service
from Products.CMFPlone.interfaces import ISearchSchema
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.i18n import translate
from zope.interface import implementer
from zope.publisher.interfaces import IPublishTraverse
import json


@implementer(IPublishTraverse)
class SearchFiltersGet(Service):
    def __init__(self, context, request):
        super(SearchFiltersGet, self).__init__(context, request)

    def get_basic_data(self, item):
        import pdb

        pdb.set_trace()
        return {
            "@id": item.getURL(),
            "title": item.Title,
            "description": item.description,
            "@type": item.portal_type,
            "path": item.getPath(),
            "UID": item.UID,
        }

    def get_portal_types(self):
        ttool = api.portal.get_tool("portal_types")
        ptool = api.portal.get_tool("plone_utils")
        registry = getUtility(IRegistry)
        search_settings = registry.forInterface(ISearchSchema, prefix="plone")
        types_not_searched = search_settings.types_not_searched
        types = [
            {
                "label": translate(ttool[t].Title(), context=self.request),
                "id": t,
            }
            for t in ptool.getUserFriendlyTypes()
            if t not in types_not_searched
        ]
        return sorted(types, key=lambda k: k["label"])

    def reply(self):
        settings = api.portal.get_registry_record(
            "search_sections",
            interface=IDesignPloneSettings,
        )
        sections = []
        if settings:
            settings = json.loads(settings)
            for setting in settings:
                items = []
                for section_settings in setting.get("items", []):
                    for uid in section_settings.get("linkUrl", []):
                        try:
                            section = api.content.get(UID=uid)
                        except Unauthorized:
                            # private folder
                            continue
                        if section:
                            item_infos = getMultiAdapter(
                                (section, self.request),
                                ISerializeToJsonSummary,
                            )()
                            children = section.listFolderContents()
                            if children:
                                item_infos["items"] = []
                                for children in section.listFolderContents():
                                    item_infos["items"].append(
                                        getMultiAdapter(
                                            (children, self.request),
                                            ISerializeToJsonSummary,
                                        )()
                                    )
                            if section_settings.get("title", ""):
                                item_infos["title"] = section_settings["title"]
                            items.append(item_infos)
                if items:
                    sections.append(
                        {
                            "rootPath": setting.get("rootPath", ""),
                            "items": items,
                        }
                    )
        topics = [
            getMultiAdapter(
                (brain, self.request),
                ISerializeToJsonSummary,
            )()
            for brain in api.content.find(portal_type="Pagina Argomento")
        ]
        return {
            "sections": sections,
            "topics": topics,
            "portal_types": self.get_portal_types(),
        }



# -*- coding: utf-8 -*-
STRUTTURA = [
    {
        'id': 'amministrazione-trasparente',
        'title': 'Amministrazione Trasparente',
        'type': 'Document',
        'children': [
            {
                'id': ' Disposizioni generali',
                'title': 'Disposizioni generali',
                'type': 'Document',
                'children': [
                    {
                        'id': 'Piano triennale per la prevenzione della corruzione e della trasparenza',
                        'title': 'Piano triennale per la prevenzione della corruzione e della trasparenza',
                        'type': 'Document',
                    },
                    {
                        'id': 'Atti generali',
                        'title': 'Atti generali',
                        'type': 'Document',
                    },
                    {
                        'id': 'Oneri informativi per cittadini e imprese',
                        'title': 'Oneri informativi per cittadini e imprese',
                        'type': 'Document',
                    },
                    {
                        'id': 'Burocrazia zero',
                        'title': 'Burocrazia zero',
                        'type': 'Document',
                    },
                ],
            },
            {
                'id': 'Organizzazione',
                'title': 'Organizzazione',
                'type': 'Document',
                'children': [
                    {
                        'id': 'Titolari di incarichi politici, di amministrazione, di direzione o di governo',
                        'title': 'Titolari di incarichi politici, di amministrazione, di direzione o di governo',
                        'type': 'Document',
                    },
                    {
                        'id': 'Sanzioni per mancata comunicazione dei dati ',
                        'title': 'Sanzioni per mancata comunicazione dei dati',
                        'type': 'Document',
                    },
                    {
                        'id': 'Rendiconti gruppi consiliari regionali/provinciali',
                        'title': 'Rendiconti gruppi consiliari regionali/provinciali',
                        'type': 'Document',
                    },
                    {
                        'id': 'Articolazione degli uffici',
                        'title': 'Articolazione degli uffici',
                        'type': 'Document',
                    },
                    {
                        'id': 'Telefono e posta elettronica',
                        'title': 'Telefono e posta elettronica',
                        'type': 'Document',
                    },
                ],
            },
            {
                'id': 'Consulenti e collaboratori',
                'title': 'Consulenti e collaboratori',
                'type': 'Document',
                'children': [
                    {
                        'id': 'Titolari di incarichi di collaborazione o consulenza',
                        'title': 'Consulenti e collaboratori',
                        'type': 'Document',
                    },
                ],
            },
            {
                'id': 'Personale',
                'title': 'Personale',
                'type': 'Document',
                'children': [
                    {
                        'id': 'Titolari di incarichi dirigenziali amministrativi di vertice',
                        'title': 'Titolari di incarichi dirigenziali amministrativi di vertice',
                        'type': 'Document',
                    },
                    {
                        'id': 'Titolari di incarichi dirigenziali (dirigenti non generali)',
                        'title': 'Titolari di incarichi dirigenziali (dirigenti non generali)',
                        'type': 'Document',
                    },
                    {
                        'id': 'Dirigenti cessati',
                        'title': 'Dirigenti cessati',
                        'type': 'Document',
                    },
                    {
                        'id': 'Sanzioni per mancata comunicazione dei dati',
                        'title': 'Sanzioni per mancata comunicazione dei dati',
                        'type': 'Document',
                    },
                    {
                        'id': 'Posizioni organizzative',
                        'title': 'Posizioni organizzative',
                        'type': 'Document',
                    },
                    {
                        'id': 'Dotazione organica',
                        'title': 'Dotazione organica',
                        'type': 'Document',
                    },
                    {
                        'id': 'Personale non a tempo indeterminato',
                        'title': 'Personale non a tempo indeterminato',
                        'type': 'Document',
                    },
                    {
                        'id': 'Tassi di assenza',
                        'title': 'Tassi di assenza',
                        'type': 'Document',
                    },
                    {
                        'id': 'Incarichi conferiti e autorizzati ai dipendenti (dirigenti e non dirigenti)',
                        'title': 'Incarichi conferiti e autorizzati ai dipendenti (dirigenti e non dirigenti)',
                        'type': 'Document',
                    },
                    {
                        'id': 'Contrattazione collettiva',
                        'title': 'Contrattazione collettiva',
                        'type': 'Document',
                    },
                    {
                        'id': 'Contrattazione integrativa',
                        'title': 'Contrattazione integrativa',
                        'type': 'Document',
                    },
                    {
                        'id': 'OIV',
                        'title': 'OIV',
                        'type': 'Document',
                    },
                ],
            },
            {
                'id': 'Bandi di concorso',
                'title': 'Bandi di concorso',
                'type': 'Document',
            },
            {
                'id': 'Performance',
                'title': 'Performance',
                'type': 'Document',
                'children': [
                    {
                        'id': 'Sistema di misurazione e valutazione della Performance',
                        'title': 'Sistema di misurazione e valutazione della Performance',
                        'type': 'Document',
                    },
                    {
                        'id': 'Piano della Performance',
                        'title': 'Piano della Performance',
                        'type': 'Document',
                    },
                    {
                        'id': 'Relazione sulla Performance',
                        'title': 'Relazione sulla Performance',
                        'type': 'Document',
                    },
                    {
                        'id': 'Ammontare complessivo dei premi',
                        'title': 'Ammontare complessivo dei premi',
                        'type': 'Document',
                    },
                    {
                        'id': 'Dati relativi ai premi',
                        'title': 'Dati relativi ai premi',
                        'type': 'Document',
                    },
                    {
                        'id': 'Benessere organizzativo',
                        'title': 'Benessere organizzativo',
                        'type': 'Document',
                    },
                ],
            },
            {
                'id': 'Enti controllati',
                'title': 'Enti controllati',
                'type': 'Document',
                'children': [
                    {
                        'id': 'Enti pubblici vigilati',
                        'title': 'Enti pubblici vigilati',
                        'type': 'Document',
                    },
                    {
                        'id': 'Società partecipate',
                        'title': 'Società partecipate',
                        'type': 'Document',
                    },
                    {
                        'id': 'Enti di diritto privato controllati',
                        'title': 'Enti di diritto privato controllati',
                        'type': 'Document',
                    },
                    {
                        'id': 'Rappresentazione grafica',
                        'title': 'Rappresentazione grafica',
                        'type': 'Document',
                    },
                ],
            },
            {
                'id': 'Attività e procedimenti',
                'title': 'Attività e procedimenti',
                'type': 'Document',
                'children': [
                    {
                        'id': 'Tipologie di procedimento',
                        'title': 'Tipologie di procedimento',
                        'type': 'Document',
                    },
                    {
                        'id': 'Monitoraggio tempi procedimentali',
                        'title': 'Monitoraggio tempi procedimentali',
                        'type': 'Document',
                    },
                    {
                        'id': 'Dichiarazioni sostitutive e acquisizione d\'ufficio dei dati',
                        'title': 'Dichiarazioni sostitutive e acquisizione d\'ufficio dei dati',
                        'type': 'Document',
                    },
                ],
            },
            {
                'id': 'Provvedimenti',
                'title': 'Provvedimenti',
                'type': 'Document',
                'children': [
                    {
                        'id': 'Provvedimenti organi indirizzo politico',
                        'title': 'Provvedimenti organi indirizzo politico',
                        'type': 'Document',
                    },
                    {
                        'id': 'Provvedimenti organi indirizzo politico',
                        'title': 'Provvedimenti organi indirizzo politico',
                        'type': 'Document',
                    },
                    {
                        'id': 'Provvedimenti dirigenti amministrativi',
                        'title': 'Provvedimenti dirigenti amministrativi',
                        'type': 'Document',
                    },
                    {
                        'id': 'Provvedimenti dirigenti amministrativi',
                        'title': 'Provvedimenti dirigenti amministrativi',
                        'type': 'Document',
                    },
                ],
            },
            {
                'id': 'Controlli sulle imprese',
                'title': 'Controlli sulle imprese',
                'type': 'Document',
            },
            {
                'id': 'Bandi di gara e contratti',
                'title': 'Bandi di gara e contratti',
                'type': 'Document',
                'children': [
                    {
                        'id': 'Informazioni sulle singole procedure in formato tabellare',
                        'title': 'Informazioni sulle singole procedure in formato tabellare',
                        'type': 'Document',
                    },
                    {
                        'id': 'Atti delle amministrazioni aggiudicatrici e degli enti aggiudicatori distintamente per ogni procedura',
                        'title': 'Atti delle amministrazioni aggiudicatrici e degli enti aggiudicatori distintamente per ogni procedura',
                        'type': 'Document',
                    },
                ],
            },
            {
                'id': 'Sovvenzioni,contributi,sussidi,vantaggi economici',
                'title': 'Sovvenzioni, contributi, sussidi, vantaggi economici',
                'type': 'Document',
                'children': [
                    {
                        'id': 'Criteri e modalità',
                        'title': 'Criteri e modalità',
                        'type': 'Document',
                    },
                    {
                        'id': 'Atti di concessione',
                        'title': 'Atti di concessione',
                        'type': 'Document',
                    },
                ],
            },
            {
                'id': 'Bilanci',
                'title': 'Bilanci',
                'type': 'Document',
                'children': [
                    {
                        'id': 'Bilancio preventivo e consuntivo',
                        'title': 'Bilancio preventivo e consuntivo',
                        'type': 'Document',
                    },
                    {
                        'id': 'Piano degli indicatori e risultati attesi di bilancio',
                        'title': 'Piano degli indicatori e risultati attesi di bilancio',
                        'type': 'Document',
                    },
                ],
            },
            {
                'id': 'Beni immobili e gestione patrimonio',
                'title': 'Beni immobili e gestione patrimonio',
                'type': 'Document',
                'children': [
                    {
                        'id': 'Patrimonio immobiliare',
                        'title': 'Patrimonio immobiliare',
                        'type': 'Document',
                    },
                    {
                        'id': 'Canoni di locazione o affitto',
                        'title': 'Canoni di locazione o affitto',
                        'type': 'Document',
                    },
                ],
            },
            {
                'id': 'Controlli e rilievi sull\'amministrazione',
                'title': 'Controlli e rilievi sull\'amministrazione',
                'type': 'Document',
                'children': [
                    {
                        'id': 'Organismi indipendenti di valutazione, nuclei di valutazione o altri organismi con funzioni analoghe',
                        'title': 'Organismi indipendenti di valutazione, nuclei di valutazione o altri organismi con funzioni analoghe',
                        'type': 'Document',
                    },
                    {
                        'id': 'Organi di revisione amministrativa e contabile',
                        'title': 'Organi di revisione amministrativa e contabile',
                        'type': 'Document',
                    },
                    {
                        'id': 'Corte dei conti',
                        'title': 'Corte dei conti',
                        'type': 'Document',
                    },
                ],
            },
            {
                'id': 'Servizi erogati',
                'title': 'Servizi erogati',
                'type': 'Document',
                'children': [
                    {
                        'id': 'Carta dei servizi e standard di qualità',
                        'title': 'Carta dei servizi e standard di qualità',
                        'type': 'Document',
                    },
                    {
                        'id': 'Class Action',
                        'title': 'Class Action',
                        'type': 'Document',
                    },
                    {
                        'id': 'Costi contabilizzati',
                        'title': 'Costi contabilizzati',
                        'type': 'Document',
                    },
                    {
                        'id': 'Liste di attesa',
                        'title': 'Liste di attesa',
                        'type': 'Document',
                    },
                    {
                        'id': 'Servizi in rete',
                        'title': 'Servizi in rete',
                        'type': 'Document',
                    },
                ],
            },
            {
                'id': 'Pagamenti dell\'amministrazione',
                'title': 'Pagamenti dell\'amministrazione',
                'type': 'Document',
                'children': [
                    {
                        'id': 'Dati sui pagamenti',
                        'title': 'Dati sui pagamenti',
                        'type': 'Document',
                    },
                    {
                        'id': 'Dati sui pagamenti del servizio sanitario nazionale',
                        'title': 'Dati sui pagamenti del servizio sanitario nazionale',
                        'type': 'Document',
                    },
                    {
                        'id': 'Indicatore di tempestività dei pagamenti',
                        'title': 'Indicatore di tempestività dei pagamenti',
                        'type': 'Document',
                    },
                    {
                        'id': 'IBAN e pagamenti informatici',
                        'title': 'IBAN e pagamenti informatici',
                        'type': 'Document',
                    },
                ],
            },
            {
                'id': 'Opere pubbliche',
                'title': 'Opere pubbliche',
                'type': 'Document',
                'children': [
                    {
                        'id': 'Nuclei di valutazione e verifica degli investimenti pubblici',
                        'title': 'Nuclei di valutazione e verifica degli investimenti pubblici',
                        'type': 'Document',
                    },
                    {
                        'id': 'Atti di programmazione delle opere pubbliche',
                        'title': 'Atti di programmazione delle opere pubbliche',
                        'type': 'Document',
                    },
                    {
                        'id': 'Tempi costi e indicatori di realizzazione delle opere pubbliche',
                        'title': 'Tempi costi e indicatori di realizzazione delle opere pubbliche',
                        'type': 'Document',
                    },
                ],
            },
            {
                'id': 'Pianificazione e governo del territorio',
                'title': 'Pianificazione e governo del territorio',
                'type': 'Document',
            },
            {
                'id': 'Informazioni ambientali',
                'title': 'Informazioni ambientali',
                'type': 'Document',
            },
            {
                'id': 'Strutture sanitarie private accreditate',
                'title': 'Strutture sanitarie private accreditate',
                'type': 'Document',
            },
            {
                'id': 'Interventi straordinari e di emergenza',
                'title': 'Interventi straordinari e di emergenza',
                'type': 'Document',
            },
            {
                'id': 'Altri contenuti - Prevenzione della corruzione',
                'title': 'Prevenzione della corruzione',
                'type': 'Document',
            },
            {
                'id': 'Altri contenuti - Accesso Civico',
                'title': 'Accesso Civico',
                'type': 'Document',
            },
            {
                'id': 'Altri contenuti - Accessibilità e catalogo di dati, metadati e banche dati',
                'title': 'Accessibilità e catalogo di dati, metadati e banche dati',
                'type': 'Document',
            },
            {
                'id': 'Altri Contenuti',
                'title': 'Altri contenuti',
                'type': 'Document',
            },
        ],
    },
]



# -*- coding: utf-8 -*-
from design.plone.policy.testing import DESIGN_PLONE_POLICY_API_FUNCTIONAL_TESTING
from design.plone.policy.enabled_vocabularies import allowed_vocabularies
from plone import api
from plone.app.testing import (
    TEST_USER_ID,
    setRoles,
)
from plone.restapi.testing import RelativeSession

import unittest
import transaction


class TestVocabularyEndpointCustom(unittest.TestCase):

    layer = DESIGN_PLONE_POLICY_API_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        # create some test users
        api.user.create(
            username="member",
            email="member@example.org",
            password="secret",
            roles=("Member",),
        )
        api.user.create(
            username="contributor",
            email="contributor@example.org",
            password="secret",
            roles=("Member", "Contributor"),
        )

        api.user.create(
            username="editor",
            email="editor@example.org",
            password="secret",
            roles=("Member", "Editor", "Reviewer"),
        )

        transaction.commit()

    def test_anonymous_can_get_standard_allowed_vocabularies(self):
        api_session = RelativeSession(self.portal_url)
        api_session.headers.update({"Accept": "application/json"})
        response = api_session.get("/@vocabularies/plone.app.vocabularies.Keywords")

        self.assertEqual(response.status_code, 200)

        api_session.close()

    def test_anonymous_can_get_custom_allowed_vocabularies(self):
        api_session = RelativeSession(self.portal_url)
        api_session.headers.update({"Accept": "application/json"})
        for name in allowed_vocabularies():
            response = api_session.get("/@vocabularies/{}".format(name))

            self.assertEqual(response.status_code, 200)

        api_session.close()

    def test_anonymous_cant_get_disallowed_vocabularies(self):
        api_session = RelativeSession(self.portal_url)
        api_session.headers.update({"Accept": "application/json"})
        response = api_session.get("/@vocabularies/design.plone.vocabularies.argomenti")
        self.assertEqual(response.status_code, 401)

        api_session.close()

    def test_authenticated_can_get_custom_allowed_vocabularies(self):
        api_session = RelativeSession(self.portal_url)
        api_session.headers.update({"Accept": "application/json"})

        for username in ["member", "contributor", "editor"]:
            api_session.auth = (username, "secret")

            for name in allowed_vocabularies():
                response = api_session.get("/@vocabularies/{}".format(name))
                self.assertEqual(response.status_code, 200)

        api_session.close()

    def test_basic_users_cant_get_other_vocabularies(self):
        api_session = RelativeSession(self.portal_url)
        api_session.headers.update({"Accept": "application/json"})

        api_session.auth = ("member", "secret")
        response = api_session.get("/@vocabularies/design.plone.vocabularies.argomenti")

        self.assertEqual(response.status_code, 401)

        api_session.close()



# -*- coding: utf-8 -*-
from design.plone.policy.testing import (
    DESIGN_PLONE_POLICY_API_FUNCTIONAL_TESTING,
)
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.registry.interfaces import IRegistry
from plone.restapi.testing import RelativeSession
from Products.CMFPlone.interfaces import ISearchSchema
from transaction import commit
from zope.component import getUtility

import unittest


class SearchFiltersAPITest(unittest.TestCase):

    layer = DESIGN_PLONE_POLICY_API_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        self.api_session_anon = RelativeSession(self.portal_url)
        self.api_session_anon.headers.update({"Accept": "application/json"})

        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def tearDown(self):
        self.api_session.close()
        self.api_session_anon.close()

    def test_endpoint_exists(self):
        response = self.api_session.get("/@search-filters")

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.headers.get("Content-Type"), "application/json")

    def test_endpoint_return_list_of_sections(self):
        response = self.api_session.get("/@search-filters").json()

        self.assertIn("sections", response)
        sections = response["sections"]

        self.assertEqual(len(sections), 1)
        self.assertEqual(len(sections[0]["items"]), 4)
        self.assertEqual("Amministrazione", sections[0]["items"][0]["title"])
        self.assertEqual("Servizi", sections[0]["items"][1]["title"])
        self.assertEqual("Novità", sections[0]["items"][2]["title"])
        self.assertEqual("Documenti e dati", sections[0]["items"][3]["title"])

    def test_endpoint_return_list_of_topics_empty_if_no_topics(self):
        response = self.api_session.get("/@search-filters").json()

        self.assertIn("topics", response)
        self.assertEqual(response["topics"], [])

    def test_endpoint_return_list_of_topics_based_on_roles(self):

        api.content.create(
            container=self.portal, type="Pagina Argomento", title="private topic"
        )
        public_topic = api.content.create(
            container=self.portal, type="Pagina Argomento", title="public topic"
        )
        api.content.transition(obj=public_topic, transition="publish")
        commit()

        response = self.api_session.get("/@search-filters").json()
        self.assertIn("topics", response)
        self.assertEqual(len(response["topics"]), 2)

        response = self.api_session_anon.get("/@search-filters").json()

        self.assertIn("topics", response)
        self.assertEqual(len(response["topics"]), 1)

    def test_endpoint_return_list_of_searchable_types(self):
        response = self.api_session.get("/@search-filters").json()

        self.assertIn("portal_types", response)
        types = [x["id"] for x in response["portal_types"]]
        self.assertIn("Document", types)

        registry = getUtility(IRegistry)
        settings = registry.forInterface(ISearchSchema, prefix="plone")
        settings.types_not_searched = tuple(
            list(settings.types_not_searched) + ["Document"]
        )
        commit()

        response = self.api_session.get("/@search-filters").json()

        self.assertIn("portal_types", response)
        types = [x["id"] for x in response["portal_types"]]
        self.assertNotIn("Document", types)



from Products.Five.browser import BrowserView
from Products.CMFCore.WorkflowCore import WorkflowException
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import _createObjectByType
from zope.component import queryUtility
from plone.i18n.normalizer.interfaces import IURLNormalizer
from plone.protect.interfaces import IDisableCSRFProtection
from zope.interface import alsoProvides
from design.plone.policy.browser.config import STRUTTURA
from uuid import uuid4

import logging

logger = logging.getLogger("trasparenza")


def set_property(context, prop_name, value):
    if context.hasProperty(prop_name):
        context.manage_changeProperties(prop_name=value)
    else:
        context.manage_addProperty(prop_name, value, 'string')


def setAllowedContents(content, allowed_ct):
    content.setConstrainTypesMode(1)
    content.setLocallyAllowedTypes(allowed_ct)
    content.setImmediatelyAddableTypes(allowed_ct)


def set_layout(context, layout_name):
    if context.hasProperty('default_page'):
        context.manage_delProperties(['default_page'])
    set_property(context, 'layout', layout_name)


class CreaTrasparenza(BrowserView):
    def __call__(self):
        parent = self.context
        try:
            children = self.context.struttura_trasparenza()
        except Exception:
            children = STRUTTURA
        alsoProvides(self.request, IDisableCSRFProtection)
        _createObjects(parent, children)


def _createObjects(parent, children):

    wftool = getToolByName(parent, "portal_workflow")

    for new_obj in children:
        logger.info("Processing object: %s" % new_obj['id'])

        #            if new_obj.get('lang', '') == 'neutral':
        #                language = ''
        #            else:
        #                language = 'it'

        local_parent = parent
        existing = parent.objectIds()
        new_obj['id'] = queryUtility(IURLNormalizer).normalize(new_obj['id'])
        if 'remove' in new_obj:
            logger.info("Removing content: %s" % new_obj['id'])
            if new_obj['id'] in existing:
                local_parent.manage_delObjects(
                    [
                        new_obj['id'],
                    ]
                )
            else:
                logger.info("Content to be removed does not exists: %s" % new_obj['id'])
            continue
        elif new_obj['id'] not in existing:
            _createObjectByType(
                new_obj['type'],
                local_parent,
                id=new_obj['id'],
                title=new_obj.get('title', ''),
                description=new_obj.get('description', ''),
            )
            logger.info("Creating content: %s" % new_obj['id'])

        else:
            logger.info("%s already exists" % new_obj['id'])
        logger.info("Now to modify the new_obj...")
        obj = local_parent.get(new_obj['id'], None)
        if obj is None:
            logger.info("can't get new_obj %s to modify it!" % new_obj['id'])
            continue
        #            obj.setLanguage(language)
        if obj.portal_type != new_obj['type']:
            logger.error("types don't match!")
            continue
        if 'layout' in new_obj:
            set_layout(obj, new_obj['layout'])
        if 'workflow_transition' in new_obj:
            try:
                wftool.doActionFor(obj, new_obj['workflow_transition'])
            except WorkflowException:
                logger.warning("couldn't do workflow transition")
        # if 'exclude_from_nav' in new_obj:
        #     obj.setExcludeFromNav(new_obj['exclude_from_nav'])
        if 'allowed_types' in new_obj:
            setAllowedContents(obj, new_obj['allowed_types'])

        data = generate_listing_query(obj)
        obj.blocks = data['blocks']
        obj.blocks_layout = data['blocks_layout']

        obj.reindexObject()
        #            if language == 'it':
        #                kwargs = {}
        #                title = new_obj.get('en_title', None)
        #                if title:
        #                    kwargs['title'] = title
        #                _id = new_obj.get('en_id', None)
        #                if _id:
        #                    kwargs['id'] = _id
        #                try:
        #                    obj.addTranslation('en', **kwargs)
        #                except (AlreadyTranslated, BadRequest):
        #                    pass

        children = new_obj.get('children', [])
        if len(children) > 0:
            _createObjects(obj, children)
        # this needs to be done after children creation
        if 'default_page' in new_obj and new_obj['default_page'] in obj.objectIds():
            obj.setDefaultPage(new_obj['default_page'])


def generate_listing_query(obj):
    title_uuid = str(uuid4())
    listing_uuid = str(uuid4())
    data = {}
    query = [
        {
            "i": "path",
            "o": "plone.app.querystring.operation.string.path",
            "v": "{uid}::1".format(uid=getattr(obj, '_plone.uuid')),
        }
    ]
    data["blocks"] = {
        title_uuid: {"@type": "title"},
        listing_uuid: {
            "@type": "listing",
            "block": listing_uuid,
            "query": [],
            "querystring": {
                "query": query,
                "sort_on": obj.get("sort_on", "getObjPositionInParent"),
                "sort_order": obj.get("sort_reversed", False),
                "b_size": obj.get("item_count", "30"),
            }
        },
    }
    data["blocks_layout"] = {"items": [title_uuid, listing_uuid]}
    return data



# -*- coding: utf-8 -*-
from copy import deepcopy
from design.plone.policy.interfaces import IDesignPlonePolicySettings
from design.plone.policy.setuphandlers import disable_searchable_types
from design.plone.policy.setuphandlers import set_default_subsite_colors
from plone import api
from plone.app.upgrade.utils import installOrReinstallProduct
from Products.CMFPlone.interfaces import ISelectableConstrainTypes

import logging


logger = logging.getLogger(__name__)

DEFAULT_PROFILE = "profile-design.plone.policy:default"


def update_profile(context, profile, run_dependencies=True):
    context.runImportStepFromProfile(DEFAULT_PROFILE, profile, run_dependencies)


def update_types(context):
    update_profile(context, "typeinfo")


def update_rolemap(context):
    update_profile(context, "rolemap")


def update_registry(context):
    update_profile(context, "plone.app.registry", run_dependencies=False)


def update_catalog(context):
    update_profile(context, "catalog")


def update_controlpanel(context):
    update_profile(context, "controlpanel")


def to_1200(context):
    def fix_field_name(blocks):
        """ """
        found = False
        for block in blocks.values():
            if block.get("@type", "") == "form" and block.get("to", ""):
                block["default_to"] = block.get("to", "")
                del block["to"]
                found = True
        return found

    installOrReinstallProduct(api.portal.get(), "collective.volto.formsupport")
    logger.info("Changing form block fields.")
    i = 0
    brains = api.content.find(object_provides="plone.restapi.behaviors.IBlocks")
    tot = len(brains)
    fixed_items = []
    for brain in brains:
        i += 1
        if i % 1000 == 0:
            logger.info("Progress: {}/{}".format(i, tot))
        item = brain.getObject()
        blocks = deepcopy(getattr(item, "blocks", {}))
        if blocks:
            to_update = fix_field_name(blocks)
            if to_update:
                item.blocks = blocks
                fixed_items.append(brain.getPath())

    logger.info("Finish")
    if fixed_items:
        logger.info("Updated items:")
        for fixed in fixed_items:
            logger.info("- {}".format(fixed))
    else:
        logger.info("No items affected.")


def to_1300(context):
    disable_searchable_types()


def to_1400(context):
    old = api.portal.get_registry_record(name="design.plone.policy.twitter_token")
    context.runAllImportStepsFromProfile("profile-design.plone.policy:to_1400")
    update_registry(context)

    if old:
        api.portal.set_registry_record(
            "twitter_token", old, interface=IDesignPlonePolicySettings
        )


def to_1500(context):
    """This upgrade handles that a type "Bando" is now  addmitted by default
    inside the folder "Documenti e Dati".
    This method just ADD THE CONTENT TYPE to the current list of types that you
    can add inside that folder only if it's not already there.
    tp#17807
    """

    doc_brains = api.content.find(portal_type="Document")
    doc_e_dati_list = [x for x in doc_brains if x.Title == "Documenti e dati"]
    if len(doc_e_dati_list) == 1:
        doc_e_dati = doc_e_dati_list[0].getObject()

        constraints = ISelectableConstrainTypes(doc_e_dati)
        allowed_types = constraints.getLocallyAllowedTypes()
        if "Bando" not in allowed_types:
            allowed_types.append("Bando")
            constraints.setLocallyAllowedTypes(allowed_types)

            logger.info("Enabled 'Bando' inside 'Ducumenti e dati' folder.")
        else:
            logger.info(
                "'Bando' already enabled in 'Ducumenti e dati' folder,"
                " not changes needed."
            )
    else:
        logger.warning(
            "More than one Document with title 'Documenti e dati'. "
            "Type 'Bando' inside 'Ducumenti e dati' folder not "
            "enabled."
        )


def to_1600(context):
    installOrReinstallProduct(api.portal.get(), "collective.volto.subsites")
    set_default_subsite_colors()
    logger.info("### CHANGE SUBSITE COLOR light-blue => teal")
    subsites = api.content.find(portal_type="Subsite")
    for brain in subsites:
        subsite = brain.getObject()
        if getattr(subsite, "subsite_css_class", "") == "light-blue":
            subsite.subsite_css_class = "teal"
            logger.info("- {}".format(brain.getURL()))


def to_1700(context):
    installOrReinstallProduct(api.portal.get(), "collective.volto.subfooter")



# -*- coding: utf-8 -*-
from copy import deepcopy
from plone import api
from plone.memoize import ram
from plone.restapi.search.utils import unflatten_dotted_dict
from plone.restapi.services import Service
from time import time
from urllib.parse import urlencode
from zope.interface import implementer
from zope.publisher.interfaces import IPublishTraverse
from design.plone.policy.interfaces import IDesignPlonePolicySettings

import logging
import requests
import six

logger = logging.getLogger(__name__)
ENDPOINT = "https://api.twitter.com/2/tweets/search/recent"


def _feed_cachekey(method, self, query):
    """
    method for ramcache that store time and query.
    Cache time is 30 minutes
    """
    timestamp = time() // (60 * 30 * 1)
    return "{timestamp}:{query}".format(
        timestamp=timestamp, query=urlencode(query)
    )


@implementer(IPublishTraverse)
class TwitterFeedGet(Service):
    def reply(self):

        token = api.portal.get_registry_record(
            "twitter_token", interface=IDesignPlonePolicySettings
        )
        if not token:
            msg = "BEARER token not set: please set it into plone registry to be able to fetch Tweets."  # noqa
            logger.error(msg)
            self.request.response.setStatus(500)
            return dict(error=dict(message=msg))

        query = self.generate_query()
        if query.get("error", ""):
            self.request.response.setStatus(400)
            return dict(error=dict(message=query["error"]))
        try:
            res = self.retrieve_tweets(query=query)
        except Exception as e:
            logger.exception(e)
            self.request.response.setStatus(500)
            msg = getattr(e, "message", e.__str__())
            return dict(error=dict(message=msg))
        return res

    @ram.cache(_feed_cachekey)
    def retrieve_tweets(self, query):
        token = api.portal.get_registry_record(
            "twitter_token", interface=IDesignPlonePolicySettings
        )

        resp = requests.get(
            url=ENDPOINT,
            params=query,
            headers={"Authorization": "Bearer {}".format(token)},
        )
        # raise an exception if resp is not successful
        resp.raise_for_status()
        return self.convert_tweets(data=resp.json())

    def generate_query(self):
        query = self.request.form.copy()
        query = unflatten_dotted_dict(query)
        if not query:
            return {"error": "You need to provide at least an author."}
        max_results = int(query.get("max_results", "10"))
        if max_results < 10 or max_results > 100:
            return {"error": "max_results should be between 10 and 100."}
        authors = query.get("authors", [])
        if not authors:
            return {"error": "You need to provide at least an author."}
        if isinstance(authors, six.string_types):
            authors = authors.split(",")
        res = {
            "query": "from: {}".format(" OR ".join(authors)),
            # additional infos for tweets
            "tweet.fields": "entities,source,public_metrics,created_at",
            "expansions": "attachments.media_keys,author_id",
            "media.fields": "type,preview_image_url,height,media_key,public_metrics,url,width",  # noqa
            "user.fields": "name,profile_image_url,username",
        }
        return res

    def convert_tweets(self, data):
        tweets = data.get("data", [])
        users = data.get("includes", {}).get("users", [])

        if not tweets:
            return []

        res = []
        for tweet in tweets:
            html = self.convert_data_to_html(data=tweet)
            if html:
                tweet_data = {
                    "text": html,
                    "id": tweet["id"],
                    "created_at": tweet["created_at"],
                    "retweet_count": tweet["public_metrics"]["retweet_count"],
                    "reply_count": tweet["public_metrics"]["reply_count"],
                    "like_count": tweet["public_metrics"]["like_count"],
                }
                author_id = tweet.get("author_id", "")
                for user in users:
                    if author_id == user["id"]:
                        tweet_data["author"] = user
                res.append(tweet_data)
        return res

    def convert_data_to_html(self, data):
        entities = data.get("entities", {})
        text = data.get("text")
        if not entities:
            return text
        html = deepcopy(text)
        href_template = (
            '<a href="{url}" title="{title}" target="_blank">{text}</a>'
        )
        #  replace hashtags
        for hashtag in entities.get("hashtags", []):
            tag_text = text[hashtag["start"] : hashtag["end"]]  # noqa
            replaced = href_template.format(
                url="https://twitter.com/hashtag/{}".format(hashtag["tag"]),
                title=tag_text,
                text=tag_text,
            )
            html = html.replace(tag_text, replaced,)

        # replace mentions
        for mention in entities.get("mentions", []):
            mention_text = text[mention["start"] : mention["end"]]  # noqa
            replaced = href_template.format(
                url="https://twitter.com/{}".format(mention["username"]),
                title=mention_text,
                text=mention_text,
            )
            html = html.replace(mention_text, replaced,)

        #  replace urls
        occurrences = {}
        for url_data in entities.get("urls", []):
            url = url_data["url"]
            if url not in occurrences:
                occurrences[url] = 1
            else:
                occurrences[url] += 1
        for url_data in entities.get("urls", []):
            url = url_data["url"]
            to_replace = text[url_data["start"] : url_data["end"]]  # noqa
            if occurrences[url] > 1:
                html = html.replace(to_replace, "")
            else:
                if url_data["display_url"].startswith("pic.twitter.com"):
                    html = html.replace(to_replace, "")
                else:
                    replaced = href_template.format(
                        url=url,
                        title=url_data.get("title", url_data["display_url"]),
                        text=url_data["display_url"],
                    )
                    html = html.replace(to_replace, replaced,)
        return html
